import * as _ from "lodash";
import { autorun } from "mobx";
import { CourseClass } from "../models";
import { BaseAction, BaseActionProps, BaseActionState } from "./BaseAction";

export interface ClassSetterProps extends BaseActionProps {
	classNo?: number;
}

export interface ClassSetterState extends BaseActionState {}

export class ClassSetter<
	P extends ClassSetterProps = ClassSetterProps,
	S extends ClassSetterState = ClassSetterState
> extends BaseAction<P, S> {
	protected observer = autorun(() => this.handleCourseChange());

	public componentWillUnmount() {
		this.observer();
		this.store.courseState.setCurrentClass({
			isLoading: false,
			value: undefined
		});
	}

	public componentDidMount() {
		this.handleCourseChange();
	}

	public componentDidUpdate() {
		this.handleCourseChange();
	}

	protected handleCourseChange() {
		const { classNo } = this.props;
		const { courseState } = this.store;
		const { currentCourse } = courseState;

		let courseClass: CourseClass | undefined;

		if (
			!_.isNil(classNo) &&
			!_.isNil(currentCourse.value) &&
			!_.isNil(currentCourse.value.classes)
		) {
			courseClass = currentCourse.value.classes.find(
				item => item.code === classNo
			);
		}

		courseState.setCurrentClass({ value: courseClass });
	}
}
