import * as _ from "lodash";
import * as Models from "../models";
import { BaseAction, BaseActionProps, BaseActionState } from "./BaseAction";

export interface ClassesFetcherProps extends BaseActionProps {
	courseCode: string;
}

export interface ClassesFetcherState extends BaseActionState {}

export class ClassesFetcher<
	P extends ClassesFetcherProps = ClassesFetcherProps,
	S extends ClassesFetcherState = ClassesFetcherState
> extends BaseAction<P, S> {
	constructor(props: P) {
		super(props);

		this.fetchClasses = this.fetchClasses.bind(this);
	}

	public componentDidMount() {
		this.fetchClasses();
	}

	public componentWillUnmount() {
		this.store.courseState.setCurrentCourse({
			isLoading: false,
			value: undefined
		});
	}

	public fetchClasses() {
		const { courseState } = this.store;
		const { currentCourse } = courseState;

		const { courseCode } = this.props;

		if (
			(_.isNil(currentCourse.value) ||
				currentCourse.value.code !== courseCode) &&
			!currentCourse.isLoading
		) {
			const course = new Models.Course();
			course.code = courseCode;
			courseState
				.fetchCourse({
					course
				})
				.then(response => response);
		}
	}
}
