import assign from "lodash/assign";
import isNil from "lodash/isNil";
import { action, observable } from "mobx";
import * as Models from "../models";
import { Loadable, State } from "./State";
import {
	Course,
	FetchCourseRequest,
	FetchCourseResponse,
	FetchCoursesResponse
} from "./static/Course";

export class CourseState extends State {
	@observable
	public courses: Loadable<Models.Course[] | undefined> = {
		isLoading: false,
		value: undefined
	};

	@observable
	public currentCourse: Loadable<Models.Course | undefined> = {
		isLoading: false,
		value: undefined
	};

	@observable
	public currentClass: Loadable<Models.CourseClass | undefined> = {
		isLoading: false,
		value: undefined
	};

	@observable
	public currentClassVideoURL: string | undefined = undefined;

	@action
	public setCurrentClass(
		currentClass: Partial<Loadable<Models.CourseClass | undefined>>
	) {
		assign(this.currentClass, currentClass);
	}

	@action
	public setCourses(courses: Partial<Loadable<Models.Course[] | undefined>>) {
		assign(this.courses, courses);
	}

	@action
	public setCurrentCourse(
		course: Partial<Loadable<Models.Course | undefined>>
	) {
		assign(this.currentCourse, course);
	}

	@action
	public setCurrentClassVideoURL(currentClassVideoURL: string | undefined) {
		this.currentClassVideoURL = currentClassVideoURL;
	}

	@action
	public async fetchCourses(): Promise<FetchCoursesResponse> {
		this.courses.isLoading = true;

		return Course.fetchCourses().then(response => {
			if (!isNil(response.courses))
				response.courses = response.courses.sort(this.compare);

			this.setCourses({
				isLoading: false,
				value: response.courses
			});

			return response;
		});
	}

	@action
	public async fetchCourse(
		request: FetchCourseRequest
	): Promise<FetchCourseResponse> {
		this.currentCourse.isLoading = true;

		return Course.fetchCourse(request).then(response => {
			this.setCurrentCourse({
				isLoading: false,
				value: response.course
			});

			return response;
		});
	}

	protected compare(c1: Models.Course, c2: Models.Course): number {
		const aValue = c1.name.toUpperCase();
		const bValue = c2.name.toUpperCase();

		return aValue > bValue ? 1 : aValue < bValue ? -1 : 0;
	}
}
