import assign from "lodash/assign";
import { action, observable } from "mobx";
import * as Models from "../models";
import { Loadable, State } from "./State";
import { FAQ, FetchFAQsResponse } from "./static/FAQ";

export class FAQState extends State {
	@observable
	public faqs: Loadable<Models.FAQ[] | undefined> = {
		isLoading: false,
		value: undefined
	};

	@action
	public setFAQs(faqs: Partial<Loadable<Models.FAQ[] | undefined>>) {
		assign(this.faqs, faqs);
	}

	@action
	public async fetchFAQs(): Promise<FetchFAQsResponse> {
		this.faqs.isLoading = true;

		return FAQ.fetchFAQs().then(response => {
			this.setFAQs({
				isLoading: false,
				value: response.faqs
			});

			return response;
		});
	}
}
