import assign from "lodash/assign";
import { action, observable } from "mobx";
import * as Models from "../models";
import { Loadable, State } from "./State";
import {
	FetchUpdatesRequest,
	FetchUpdatesResponse,
	Update
} from "./static/Update";

export class UpdateState extends State {
	@observable
	public updates: Loadable<Models.Update[] | undefined> = {
		isLoading: false,
		value: undefined
	};

	@action
	public setUpdates(updates: Partial<Loadable<Models.Update[] | undefined>>) {
		assign(this.updates, updates);
	}

	@action
	public async fetchUpdates(
		request: FetchUpdatesRequest
	): Promise<FetchUpdatesResponse> {
		this.updates.isLoading = true;

		return Update.fetchUpdates(request).then(response => {
			this.setUpdates({
				isLoading: false,
				value: response.updates
			});

			return response;
		});
	}
}
