import isNil from "lodash/isNil";
import * as Models from "../../models";
import { Base, BaseRequest, BaseResponse } from "./Base";

export class Course extends Base {
	public static async fetchCourses(): Promise<FetchCoursesResponse> {
		const { axios } = this;

		return axios
			.get(`https://open.fing.edu.uy/data/courses.json`)
			.then(httpResponse => {
				const response = this.processResponse<FetchCoursesResponse>(
					{},
					httpResponse
				);

				response.courses =
					isNil(httpResponse.data) || isNil(httpResponse.data.courses)
						? []
						: httpResponse.data.courses.map((course: any) =>
								Models.Course.fromJSON(course)
						  );

				return response;
			});
	}

	public static async fetchCourse(
		request: FetchCourseRequest
	): Promise<FetchCourseResponse> {
		const { axios } = this;
		const { course } = request;

		return axios
			.get(
				`https://open.fing.edu.uy/data/${
					course instanceof Models.Course ? course.code : course
				}.json`
			)
			.then(httpResponse => {
				const response = this.processResponse<FetchCourseResponse>(
					request,
					httpResponse
				);

				const fetchedCourse = isNil(httpResponse.data)
					? undefined
					: Models.Course.fromJSON(httpResponse.data);

				if (!isNil(fetchedCourse)) response.course = fetchedCourse;

				return response;
			});
	}
}

export interface FetchCoursesResponse extends BaseResponse {
	courses: Models.Course[];
}

export interface FetchCourseRequest extends BaseRequest {
	course: Models.Course | string;
}

export interface FetchCourseResponse extends BaseResponse {
	course: Models.Course;
}
